@@ -3,6 +3,7 @@ from __future__ import unicode_literals
 
 MESSAGES = {
     "%d min remaining to read": "zostáva %d minút na čítanie",
+    "(active)": "",
     "Also available in:": "Tiež dostupné v:",
     "Archive": "Archív",
     "Categories": "Kategórie",
@@ -19,16 +20,20 @@ MESSAGES = {
     "Posted:": "Zverejnené:",
     "Posts about %s": "Príspevky o %s",
     "Posts for year %s": "Príspevky z roku %s",
+    "Posts for {month} {day}, {year}": "Príspevky z dňa {day}. {month} {year}",
     "Posts for {month} {year}": "Príspevky za mesiac {month} z roku {year}",
     "Previous post": "Predchádzajúci príspevok",
     "Publication date": "Dátum zverejnenia",
     "RSS feed": "RSS kanál",
     "Read in English": "Čítať v slovenčine",
     "Read more": "Čítať ďalej",
-    "Skip to main content": "",
+    "Skip to main content": "Skočiť na hlavný obsah",
     "Source": "Zdroj",
+    "Subcategories:": "",
     "Tags and Categories": "Štítky a kategórie",
     "Tags": "Štítky",
+    "Write your page here.": "",
+    "Write your post here.": "",
     "old posts, page %d": "staré príspevky, strana %d",
     "page %d": "stránka %d",
 }
