@@ -3,6 +3,7 @@ from __future__ import unicode_literals
 
 MESSAGES = {
     "%d min remaining to read": "%d منٹ کا مطالعہ باقی",
+    "(active)": "",
     "Also available in:": "ان زبانوں میں بھی دستیاب:",
     "Archive": "آرکائیو",
     "Categories": "زمرے",
@@ -19,16 +20,20 @@ MESSAGES = {
     "Posted:": "اشاعت:",
     "Posts about %s": "%s کے بارے میں تحاریر",
     "Posts for year %s": "سال %s کی تحاریر",
+    "Posts for {month} {day}, {year}": "{day} {month}، {year} کی تحاریر",
     "Posts for {month} {year}": "{month} {year} کی تحاریر",
     "Previous post": "پچھلی تحریر",
     "Publication date": "تاریخِ اشاعت",
     "RSS feed": "آر ایس ایس فیڈ",
-    "Read in English": "اردو میں پڑھیے",
-    "Read more": "مزید پڑھیے",
-    "Skip to main content": "",
+    "Read in English": "اردو میں پڑھیں",
+    "Read more": "مزید پڑھیں",
+    "Skip to main content": "مرکزی متن پر جائیں",
     "Source": "سورس",
+    "Subcategories:": "ذیلی زمرے",
     "Tags and Categories": "ٹیگز اور زمرے",
     "Tags": "ٹیگز",
+    "Write your page here.": "",
+    "Write your post here.": "",
     "old posts, page %d": "پرانی تحاریر صفحہ %d",
     "page %d": "صفحہ %d",
 }
