@@ -22,6 +22,7 @@ dumb_replacements = [
 ]
 
 dumber_replacements = [
+    ['<%! import json %>\n\n', ''],
     ["<html\n\\", "<html\n"],
     ["\n'\\\n", "\n'\n"],
     ["{% endif %}\n\\", "{% endif %}\n"]
@@ -76,15 +77,18 @@ def jinjify(in_theme, out_theme):
     if child in mappings:
         parent = mappings[child]
 
-    with open(os.path.join(out_theme, "parent"), "wb+") as outf:
-        outf.write(parent + '\n')
+    with io.open(os.path.join(out_theme, "parent"), "w+", encoding='utf-8') as outf:
+        outf.write(u'{0}\n'.format(parent))
 
-    with open(os.path.join(out_theme, "engine"), "wb+") as outf:
-        outf.write("jinja\n")
+    with io.open(os.path.join(out_theme, "engine"), "w+", encoding='utf-8') as outf:
+        outf.write(u"jinja\n")
 
-    # Copy assets
-    # shutil.rmtree(os.path.join(out_theme, "assets"))
-    # shutil.copytree(os.path.join(in_theme, "assets"), os.path.join(out_theme, "assets"))
+    # Copy assets in bootstrap/bootstrap3
+    if child in ('bootstrap-jinja', 'bootstrap3-jinja'):
+        shutil.rmtree(os.path.join(out_theme, "assets"))
+        shutil.copytree(
+            os.path.join(in_theme, "assets"), os.path.join(out_theme, "assets"),
+            symlinks=True)
 
     # Copy bundles
     # shutil.copy(os.path.join(in_theme, "bundles"), os.path.join(out_theme, "bundles"))
